<?php
/**
 *
 */

/**
 *
 */
class WebpayNormalService {
  protected $errors = array();

  protected $config;

  protected $webpayTransactionService;

  protected $commerceSystem;

  /**
   *
   * @param object $config
   * @param string $commerceSystem
   */
  public function __construct($config, $commerceSystem = '') {
    $this->config = $config;
    $this->webpayTransactionService = new WebpayTransactionService($config, array('trace' => TRUE));
    $this->commerceSystem = $commerceSystem;
  }

  /**
   *
   * @param int $buyOrder
   * @param float $amount
   * @param string $finalUrl
   * @param mixed $session
   * @return mixed
   */
  public function initTransaction($buyOrder, $amount, $finalUrl, $instanceId, $session = NULL) {
    if (is_null($session)) {
      global $user;

      $session = $user->uid;
    }
    $variables = array(
      'wsInitTransactionInput' => array(
        'wSTransactionType' => 'TR_NORMAL_WS',
        'buyOrder' =>  $buyOrder,
        'sessionId' => $this->commerceSystem . ':' . $session,
        'returnURL' => url('webpay/return/' . $instanceId . '/' . $this->config->id, array('absolute' => TRUE)),
        'finalURL' => $finalUrl,
        'transactionDetails' => array(
          'amount' => $amount,
          'commerceCode' => $this->config->commerce_code,
          'buyOrder' => $buyOrder,
        ),
      ),
    );

    return $this->execute('initTransaction', $variables);
  }

  /**
   *
   * @param string $token
   *
   * @return mixed
   */
  public function getTransactionResult($token) {
    $variables = array(
      'tokenInput' => $token,
    );

    $response = $this->execute('getTransactionResult', $variables);
    if ($response) {
      $isValid = $this->execute('acknowledgeTransaction', $variables);
      if ($isValid) {
        return $response;
      }
      else {
        $this->addError('acknowledgeTransaction', t('The transaction is not valid. Check the keys.'));
      }
    }
    $this->addError('getTransactionResult', t('The transaction is not valid. Check the keys.'));

    return FALSE;
  }

  /**
   * Execute the SOAP services.
   *
   * @param string $operation
   *   The operation of the services.
   * @param array $variables
   *   The variables to send.
   *
   * @return mixed
   */
  protected function execute($operation, $variables) {
    $response = FALSE;
    try {
      $response = $this->webpayTransactionService->execute($operation, $variables);
    }
    catch (SoapFault $e) {
      $this->addError('soap', $e->getMessage());
      watchdog('Webpay SOAP', 'Soap Error: %message', array('%message' => $e->getMessage()), WATCHDOG_ERROR);
    }

    return $response;
  }

  public function getErrors($type = NULL) {
    if ($type && isset($this->errors[$type])) {
      $errors = $this->errors[$type];
    }
    else {
      $errors = $this->errors;
    }
    $this->clearErrors($type);

    return $errors;
  }


  protected function addError($type, $message) {
    $this->errors[$type][] = $message;
  }


  protected function clearErrors($type = NULL) {
    if ($type && isset($this->errors[$type])) {
      unset($this->errors[$type]);
    }
    else {
      $this->errors = array();
    }

    return $this;
  }
}
