<?php

/**
 * @file
 *
 */

/**
 *
 */
abstract class WebpaySoap extends SoapClient {
  private $useSSL = false;
  private $privateKey = "";
  private $publicCert = "";
  private $webpay_cert = "";

  protected $environment = 0;

  /**
   * Construct.
   */
  public function __construct($config, array $options)
  {
    $this->environment = $config->environment;
    $wsdl = $this->setupWsdl();


    $locationparts = parse_url($wsdl);
    $this->useSSL = $locationparts['scheme'] == "https" ? true : false;
    $this->privateKey = $config->commerce_keys['private_key'];
    $this->publicCert = $config->commerce_keys['public_key'];
    $this->webpay_cert = $config->commerce_keys['webpay_key'];

    parent::__construct($wsdl, $options);
  }


  /**
   * Helper function
   */
  protected function parameterParse(array $input, $arguments) {
    $parameters_object = new stdClass();
    foreach ($input as $key => $parameter) {
      if (!isset($arguments->{$key})) {
        $arguments->{$key} = $parameter;
      }
      if (is_array($parameter)) {
        $parameters_object->{$key} = $this->parameterParse($parameter, $arguments->{$key});
      }
      else {
        $parameters_object->{$key} = $arguments->{$key};
      }
    }

    return $parameters_object;
  }

  /**
   * Helper function
   */
  private function _arrayToObject(array $data) {
    $object = new stdClass();
    foreach ($data as $key => $value) {
      if (is_array($value)) {
        $object->{$key} = $this->_arrayToObject($value);
      }
      else {
        $object->{$key} = $value;
      }
    }

    return $object;
  }

  /**
   * Check the tree object and return the results.
   */
  private function mapping($map, $results) {
    $current = $results;
    if (!empty($map)) {
      foreach ($map as $key) {
        if (!isset($current->{$key})) {
          return NULL;
        }
        $current = $current->{$key};
      }
    }

    return $current;
  }

  /**
   * Execute the operation and mapping the results.
   */
  public function execute($operation, array $arguments = array()) {
    $arguments_object = $this->_arrayToObject($arguments);
    $input = $this->mappingInput();
    $output = $this->mappingOutput();
    $parameters = $this->parameterParse($input[$operation], $arguments_object);
    $results = $this->__soapCall($operation, array($parameters));
    $xmlResponse = $this->__getLastResponse();

    if (!libraries_load('xmlseclibs')) {
      return FALSE;
    }
    require_once "WebpayXMLSecurityDSig.php";
    $soapValidation = new SoapValidation($xmlResponse, $this->webpay_cert);
    $validationResult = $soapValidation->getValidationResult();

    if ($validationResult === TRUE) {
      return isset($results->return) ? $results->return : TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * Override.
   */
  public function __soapCall($function_name, $arguments, $options = NULL, $input_headers = NULL, &$output_headers = NULL) {

    return parent::__soapCall($function_name, $arguments, $options, $input_headers, $output_headers);
  }



  private function soapVar($parameter) {
    $value = $this->{$parameter};

    return new SoapVar($value, XSD_STRING, null, null, $parameter, self::HEADER_NS);
  }

  function __doRequest($request, $location, $saction, $version, $one_way = 0) {

    if ($this->useSSL) {
      $locationparts = parse_url($location);
      $location = 'https://';
      if (isset($locationparts['host'])) {
        $location .= $locationparts['host'];
      }

      if (isset($locationparts['port'])) {
        $location .= ':' . $locationparts['port'];
      }

      if (isset($locationparts['path'])) {
        $location .= $locationparts['path'];
      }

      if (isset($locationparts['query'])) {
        $location .= '?' . $locationparts['query'];
      }

    }
    $doc = new DOMDocument('1.0');

    $doc->loadXML($request);
    $objWSSE = new WSSESoap($doc);
    if (!libraries_load('xmlseclibs')) {
      return FALSE;
    }
    $objKey = new XMLSecurityKey(XMLSecurityKey::RSA_SHA1, array(
      'type' => 'private'
    ));
    try {
      $objKey->loadKey($this->privateKey, FALSE);
    }
    catch (Exception $e) {

      return FALSE;
    }

    $options = array(
      "insertBefore" => TRUE
    );
    try {
      $objWSSE->signSoapDoc($objKey, $options);
      $objWSSE->addIssuerSerial($this->publicCert);
    }
    catch (Exception $e) {

      return FALSE;
    }



    $retVal = parent::__doRequest($objWSSE->saveXML(), $location, $saction, $version);

    $_doc = new DOMDocument();
    $_doc->loadXML($retVal);

    return $_doc->saveXML();
  }
}
